/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.buttons.music

import dev.kord.common.Color
import dev.kord.core.behavior.interaction.response.DeferredPublicMessageInteractionResponseBehavior
import dev.kord.core.behavior.interaction.response.respond
import dev.kord.core.entity.Guild
import dev.kord.core.entity.User
import dev.kord.rest.builder.message.modify.actionRow
import dev.schlaubi.lavakord.audio.Link
import net.moonleay.botendo.extensions.music.components.MusicManager
import net.moonleay.botendo.util.ButtonUtil
import net.moonleay.botendo.util.MessageUtil

class PauseButton : net.moonleay.botendo.buttons.Button("btn.music.pause") {
    override suspend fun onInteraction(
        response: DeferredPublicMessageInteractionResponseBehavior,
        guild: Guild,
        user: User
    ) {
        val guildId = guild.id
        val link = net.moonleay.botendo.Bot.lava.getLink(guildId.toString())
        val player = link.player
        val voiceState = user.asMember(guildId).getVoiceStateOrNull()
        if (voiceState == null) {
            response.respond {
                this.embeds = mutableListOf(
                    MessageUtil.getEmbed(
                        Color(0xE0311A),
                        "You are not connected to a VC",
                        "Please connect to a VC",
                        user.asUser().username + "#" + user.asUser().discriminator
                    )
                )
            }
            return
        }

        val channelId = voiceState.channelId
        if (link.state == Link.State.NOT_CONNECTED) {
            response.respond {
                this.embeds = mutableListOf(
                    MessageUtil.getEmbed(
                        Color(0xE0311A),
                        "Not connected",
                        "I'm not in a VC and therefore, I am not playing anything.",
                        user.asUser().username + "#" + user.asUser().discriminator
                    )
                )
            }
            return
        } else if (link.state == Link.State.CONNECTED && link.lastChannelId != channelId!!.value) {
            response.respond {
                this.embeds = mutableListOf(
                    MessageUtil.getEmbed(
                        Color(0xE0311A),
                        "You are not in my VC",
                        "We are not in the same VC and therefore, you cannot control the music",
                        user.asUser().username + "#" + user.asUser().discriminator
                    )
                )
            }
            return
        }

        player.pause(!player.paused)

        val gts = MusicManager.getGuildTrackScheduler(guild, player)
        response.respond {
            this.embeds = mutableListOf(
                MessageUtil.getEmbed(
                    Color(0x52E01A),
                    if (player.paused) "I paused" else "I'm continuing",
                    if (player.paused) "I paused the song" else "I'm continuing to play the song.",
                    user.asUser().username + "#" + user.asUser().discriminator
                )
            )


            this.actionRow {
                this.components.addAll(ButtonUtil.getMusicControllerButtons(player.paused, gts.repeating).components)
            }
        }
    }
}
