/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.data

import java.io.*
import java.util.*

object CredentialManager {
    private const val foldername = "data"
    private const val filename = "credentials.nils"
    var token: String = "empty"
    var linkip: String = "empty"
    var linkpw: String = "empty"

    ///Load the needed credentials, generate a config if there is none
    fun load() {
        val folder = File(foldername)
        if (!folder.exists()) {
            save()
            return
        }
        val configFile = File(folder, filename)
        if (!configFile.exists()) {
            save()
            return
        }
        try {
            val input: InputStream = FileInputStream(foldername + File.separator + filename)
            val prop = Properties()
            prop.load(input)
            token = if (prop.getProperty("token").equals("empty")) "empty" else prop.getProperty("token")
            linkip = if (prop.getProperty("lavaip").equals("empty")) "empty" else prop.getProperty("lavaip")
            linkpw = if (prop.getProperty("lavapw").equals("empty")) "empty" else prop.getProperty("lavapw")
            input.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    ///generate a new sample config
    private fun save() {
        val folder = File(foldername)
        if (!folder.exists()) {
            try {
                folder.mkdirs()
            } catch (e: IOException) {
                e.printStackTrace()
            }
        }
        val configFile = File(foldername + File.separator + filename)
        if (!configFile.exists()) {
            try {
                configFile.createNewFile()
            } catch (e: IOException) {
                e.printStackTrace()
            }
        }
        try {
            val output: OutputStream = FileOutputStream(foldername + File.separator + filename)
            val prop = Properties()
            prop.setProperty("token", "empty")
            prop.setProperty("lavaip", "empty")
            prop.setProperty("lavapw", "empty")
            prop.store(output, null)
            output.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }
}
