/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import dev.kord.common.Color
import dev.kord.rest.builder.message.create.actionRow
import dev.schlaubi.lavakord.audio.Link
import net.moonleay.botendo.extensions.music.components.MusicManager
import net.moonleay.botendo.util.ButtonUtil
import net.moonleay.botendo.util.MessageUtil
import net.moonleay.botendo.util.TimeUtil

class QueueExtension : Extension() {
    override val name = "queue"
    override suspend fun setup() {
        publicSlashCommand {
            name = "queue"
            description = "Show whats up next"
            this.action {
                val guildId = this.guild!!.id
                val link = net.moonleay.botendo.Bot.lava.getLink(guildId.toString())
                val player = link.player
                if (link.state == Link.State.NOT_CONNECTED) {
                    MessageUtil.sendEmbedForPublicSlashCommand(
                        this,
                        Color(0xE0311A),
                        "Not connected",
                        "I'm not in a VC and therefore, I am not playing anything."
                    )
                    return@action
                }
                val track = player.playingTrack
                if (track == null) {
                    MessageUtil.sendEmbedForPublicSlashCommand(
                        this,
                        Color(0xE0A81A),
                        "Queue empty",
                        "I'm not playing anything currently"
                    )
                    return@action
                }
                val gts = MusicManager.getGuildTrackScheduler(this.guild!!.asGuild(), player)
                val q = gts.getQueue()
                var desc =
                    """${"**" + track.title + " - " + TimeUtil.getTimeFormatedShortend(track.length.inWholeMilliseconds)} (${track.author})**""" + "\n"
                for ((i, tr) in q.withIndex()) {
                    if (i >= 14)
                        continue
                    desc += tr.info.title + " - " + TimeUtil.getTimeFormatedShortend(tr.info.length) + " (" + tr.info.author + ")\n"
                }
                this.respond {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            Color(0x52E01A),
                            "Queue",
                            desc,
                            user.asUser().username + "#" + user.asUser().discriminator
                        )
                    )

                    this.actionRow {
                        this.components.addAll(
                            ButtonUtil.getMusicControllerButtons(
                                player.paused,
                                gts.repeating
                            ).components
                        )
                    }
                }
            }
        }
    }
}
