/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import dev.kord.common.Color
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.kord.getLink
import net.moonleay.botendo.extensions.music.components.MusicManager
import net.moonleay.botendo.util.MessageUtil

class StopExtension : Extension() {
    override val name = "stop"
    override suspend fun setup() {
        publicSlashCommand {
            name = "stop"
            description = "Stop playing and start leavin'"
            this.action {
                val guildId = this.guild!!.id
                val link = net.moonleay.botendo.Bot.lava.getLink(guildId)
                val player = link.player
                val u = this.user
                val vcsUser = u.asMember(guildId).getVoiceStateOrNull()
                if (vcsUser == null) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "You are not connected to a VC",
                                "Please connect to my VC", u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }
                val channelId = vcsUser.channelId
                if (link.state == Link.State.NOT_CONNECTED) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "Not connected",
                                "I'm not in a VC and therefore, I am not playing anything.",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                } else if (link.state == Link.State.CONNECTED && link.lastChannelId != channelId!!.value) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "You are not in my VC",
                                "We are not in the same VC", u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }
                player.stopTrack()
                link.destroy()
                MusicManager.getGuildTrackScheduler(this.getGuild()!!.asGuild(), player).clear()
                this.respond {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            Color(0x52E01A),
                            "I stopped and left",
                            "just like your girlfriend",
                            u.asUser().username + "#" + u.asUser().discriminator
                        )
                    )
                }
            }
        }
    }
}
